from log import Log
import json
import sqlite3
from settings import Settings

class DB:

    def __init__(self):
        # Connect to the SQLite database (creates a new file if not exists)
        self.connection = sqlite3.connect(Settings.script_dir + '/data/data.db')

        # Create a cursor object to execute SQL queries
        self.cursor = self.connection.cursor()

        # Check tables
        self.check_tables()

    def check_column(self, table, name):
        self.cursor.execute("SELECT count(*) > 0 FROM pragma_table_info(?) WHERE name=?;", (table, name))
        col_exist = self.cursor.fetchone()
        return col_exist[0] > 0
    
    def check_tables(self):

        # engine table
        self.cursor.execute('''
            CREATE TABLE IF NOT EXISTS engine (
                id TEXT PRIMARY KEY,
                type TEXT NOT NULL,
                credentials TEXT NOT NULL DEFAULT '{}',
                data TEXT NOT NULL DEFAULT '',
                config TEXT NOT NULL DEFAULT '{}'
            )
        ''')
        self.connection.commit()
        
        '''
        # DB upgrade example
        if not self.check_column('engine', 'config'):
            self.cursor.execute("ALTER TABLE engine ADD COLUMN config TEXT NOT NULL DEFAULT '{}'")
            self.connection.commit()
        '''

        self.cursor.execute('''
            CREATE TABLE IF NOT EXISTS bridge (
                id TEXT PRIMARY KEY,
                engine1 TEXT NOT NULL,
                engine2 TEXT NOT NULL,
                active INTEGER NOT NULL DEFAULT 1,
                pairs TEXT NOT NULL DEFAULT '{}',
                FOREIGN KEY(engine1) REFERENCES engine(id),
                FOREIGN KEY(engine2) REFERENCES engine(id)
            )
        ''')
        self.connection.commit()

        self.cursor.execute('''
            CREATE TABLE IF NOT EXISTS data (
                id INTEGER PRIMARY KEY,
                bridge TEXT NOT NULL,
                item1_id TEXT NOT NULL,
                item1_timestamp TEXT NOT NULL,
                item2_id TEXT NOT NULL,
                item2_timestamp TEXT NOT NULL,
                FOREIGN KEY(bridge) REFERENCES bridge(id)
            )
        ''')
        self.connection.commit()

    def get_engines(self):
        self.cursor.execute('SELECT id,type,credentials,data,config FROM engine')
        data = self.cursor.fetchall()
        result = {}
        for item in data:
            obj = {}
            obj['id'],obj['type'],obj['credentials'],obj['data'],obj['config'] = item
            result[obj['id']] = obj

        return result

    def edit_engine_data(self, id, data):
        self.cursor.execute('''UPDATE engine SET data = ? WHERE id = ?''', (data, id))
        self.connection.commit()

    def get_bridges(self):
        self.cursor.execute('SELECT id,engine1,engine2,active,pairs FROM bridge')
        data = self.cursor.fetchall()
        result = {}
        for item in data:
            obj = {}
            obj['id'],obj['engine1'],obj['engine2'],obj['active'],obj['pairs'] = item
            result[obj['id']] = obj
               
        return result

    def get_bridge_data(self, bridge_id):
        self.cursor.execute('SELECT id,item1_id,item1_timestamp,item2_id,item2_timestamp FROM data WHERE bridge=?', (bridge_id,))
        data = self.cursor.fetchall()
        result = []
        for item in data:
            obj = {}
            obj['id'],obj['item1_id'],obj['item1_timestamp'],obj['item2_id'],obj['item2_timestamp'] = item
            result.append(obj)

        return result
    
    def delete_data_item(self, id):
        self.cursor.execute('DELETE FROM data WHERE id = ?', (id,))
        self.connection.commit()
        
    def edit_data_item(self, id, item1_timestamp, item2_timestamp):
        self.cursor.execute('''UPDATE data SET item1_timestamp = ?, item2_timestamp = ? WHERE id = ?''', (item1_timestamp, item2_timestamp, id))
        self.connection.commit()

    def add_data_item(self, bridge_id, item1_id, item1_timestamp, item2_id, item2_timestamp):
        self.cursor.execute('''INSERT INTO data (bridge, item1_id, item1_timestamp, item2_id, item2_timestamp) VALUES (?, ?, ?, ?, ?)''', (bridge_id, item1_id, item1_timestamp, item2_id, item2_timestamp))
        self.connection.commit()

    def edit_engine_item(self, id, type, credentials, config):
        self.cursor.execute('''UPDATE engine SET type = ?, credentials = ?, config = ? WHERE id = ?''', (type, json.dumps(credentials), json.dumps(config), id))
        self.connection.commit()

    def add_engine_item(self, id, type, credentials, config):
        self.cursor.execute('''INSERT INTO engine (id, type, credentials, config) VALUES (?, ?, ?, ?)''', (id, type, json.dumps(credentials), json.dumps(config)))
        self.connection.commit()

    def edit_bridge_item(self, id, engine1, engine2, active, pairs):
        self.cursor.execute('''UPDATE bridge SET engine1 = ?, engine2 = ?, active = ?, pairs = ? WHERE id = ?''', (engine1, engine2, active, json.dumps(pairs), id))
        self.connection.commit()

    def add_bridge_item(self, id, engine1, engine2, active, pairs):
        self.cursor.execute('''INSERT INTO bridge (id, engine1, engine2, active, pairs) VALUES (?, ?, ?, ?, ?)''', (id, engine1, engine2, active, json.dumps(pairs)))
        self.connection.commit()

    def provision(self, settings):

        # Get engines
        engines = self.get_engines()

        for engconf in settings.engine:

            credentials = engconf['credentials'] if 'credentials' in engconf else ''

            if not engconf['id'] in engines:
                self.add_engine_item(engconf['id'], engconf['type'], credentials, engconf['config'])
            else:
                eng = engines[engconf['id']]
                if eng['type'] != engconf['type'] or eng['credentials'] != credentials or eng['config'] != engconf['config']:
                    self.edit_engine_item(engconf['id'], engconf['type'], credentials, engconf['config'])            

        # Get bridges
        bridges = self.get_bridges()

        for bridgeconf in settings.bridge:

            if not bridgeconf['id'] in bridges:
                self.add_bridge_item(bridgeconf['id'], bridgeconf['engine1'], bridgeconf['engine2'], bridgeconf['active'], bridgeconf['pairs'])
            else:
                brg = bridges[bridgeconf['id']]
                if brg['engine1'] != bridgeconf['engine1'] or brg['engine2'] != bridgeconf['engine2'] or brg['active'] != bridgeconf['active'] or brg['pairs'] != bridgeconf['pairs']:
                    self.edit_bridge_item(bridgeconf['id'], bridgeconf['engine1'], bridgeconf['engine2'], bridgeconf['active'], bridgeconf['pairs'])
