import uuid
import urllib.parse
import requests
from icalendar import Calendar, Todo
from icalendar.cal import Component
from caldav import DAVClient

from log import Log
from engine_parent import Engine_Parent
from item import Item

class Note(Component):
    """This is the base object for an vnote file.
    """
    name = 'VNOTE'
    canonical_order = ('VERSION', 'PRODID', 'CALSCALE', 'METHOD',)
    required = ('PRODID', 'VERSION', )
    singletons = ('PRODID', 'VERSION', 'CALSCALE', 'METHOD')

class Engine_CalDAV(Engine_Parent):

    CALDAV_CONTENT_TYPE_CALENDAR = {'Content-Type': 'text/calendar'}
    CALDAV_CONTENT_TYPE_NOTE = {'Content-Type': 'text/x-vnote'}
    PROD_ID = '-//UniqTec//DataBridge 1.0.0//EN'
    
    engine_type = 'VTODO'

    def __init__(self, id, engine, db):
        Engine_Parent.__init__(self, id, engine, db)

        # Avoid 401 slowdowns
        self.auth = requests.auth.HTTPDigestAuth(username=self.credentials['user'], password=self.credentials['pass'])

    def get_content_type(self):
        if self.engine_type == 'VNOTE':
            return self.CALDAV_CONTENT_TYPE_NOTE
        else:
            return self.CALDAV_CONTENT_TYPE_CALENDAR


    def set_pair_config(self, config):

        # Check todo
        if 'type' in config:
            self.engine_type = config['type']

        # Get url
        folder = config['folder']
        self.url = self.credentials['url'] + urllib.parse.quote(folder) + '/'

        # Create davclient
        self.client = DAVClient(url=self.url, auth=self.auth)

        # Get calendar
        self.calendar = self.client.calendar(url=self.url)


    def id_to_url(self, id):

        # Create path
        return self.url + id
    
    def url_to_id(self, url):
        
        # Get last path 
        id = str(url).rsplit('/', 1)[-1]
        return id

    def list_items(self):
        
        # Get objects
        data = self.calendar.objects()
                
        items = {}
        for obj in data:
            item = Item()
            item.id = self.url_to_id(obj.url.canonical())
            item.timestamp = obj.props['{DAV:}getetag']
            items[item.id] = item

        return items
    
    def get_object_from_item(self, item):
        
        if self.engine_type == 'VNOTE':

            note = Note()

            note.add('prodid', self.PROD_ID)
            note.add('version', '1.1')

            note.add('summary', item.get_title(self.capitalize_title))
            note.add('body', item.body)

            return note

        elif self.engine_type == 'VTODO':

            cal = Calendar()
            cal.add('prodid', self.PROD_ID)
            cal.add('version', '2.0')

            todo = Todo()
            
            todo.add('summary', item.get_title(self.capitalize_title))
            todo.add('description', item.body)
            todo.add('status', 'NEEDS-ACTION')

            if item.due is not None:
                todo.add('due', item.due)

            cal.add_component(todo)

            return cal
        
        else:

            return None

    def get_item_from_object(self, obj):

        item = Item()

        try:                    
            if self.engine_type == 'VNOTE':

                item.title = obj.get("summary")
                item.body = obj.get("body")

            else:

                # Get components
                for component in obj.walk():
                    if self.engine_type == 'VTODO' and component.name == 'VTODO':

                        item.title = component.get("summary")
                        item.body = component.get("description")
                        
                        if (component.get("due")):
                            item.due = component.decoded("due")
                                
        except Exception as e:
            Log.error('engine {0} error {1}'.format(self.engine['type'], e))

        return item
    
    def delete_item(self, id):

        obj = self.client.delete(self.id_to_url(id))

    def get_item(self, id):

        # Get item
        obj = self.client.request(self.id_to_url(id))
        ical = Calendar.from_ical(obj.raw)

        # Get data
        item = self.get_item_from_object(ical)
        item.id = id

        # Get etag
        obj = self.client.propfind(self.id_to_url(id))
        tree = obj.tree
        resp = tree.find('{DAV:}response/{DAV:}propstat/{DAV:}prop/{DAV:}getetag')
        item.timestamp = resp.text
    
        return item
    
    def add_item(self, item):

        new_item = item.copy()
        new_item.id = self.url_to_id(self.url + str(uuid.uuid4()))

        cal = self.get_object_from_item(new_item)
        obj = self.client.put(self.id_to_url(new_item.id), cal.to_ical(), self.get_content_type())

        new_item.timestamp = obj.headers['etag']

        return new_item

    def edit_item(self, id, item):

        new_item = item.copy()
        new_item.id = id

        cal = self.get_object_from_item(item)
        obj = self.client.put(self.id_to_url(id), cal.to_ical(), self.get_content_type())

        new_item.timestamp = obj.headers['etag']
        
        return new_item
