import json
import argparse
import sys

from log import Log
from db import DB
from engine import Engine
from args import Args
from settings import Settings

class Sync:

    db = None
    bridges = None
    engines = None

    def __init__(self):

        # Get DB
        self.db = DB()
        
        # Provision settings
        self.db.provision(Settings)

        # Only db run
        if Args.args.db_run:
            sys.exit()
    
        # Get engines
        self.engines = self.db.get_engines()

        # Get bridges
        self.bridges = self.db.get_bridges()

        # Sync
        self.sync()
    
    def sync(self):
        
        # For each bridge
        for bridge_id in self.bridges:

            bridge = self.bridges[bridge_id]
            
            # Sync
            if bridge['active'] == 1:
                self.sync_bridge(bridge['id'], bridge['engine1'], bridge['engine2'], bridge['pairs'])

    def sync_bridge(self, bridge_id, engine1_id, engine2_id, pairs_data):

        Log.debug('syncing bridge {0}'.format(bridge_id))

        # Get db data
        data = self.db.get_bridge_data(bridge_id)
        is_slowsync = len(data)==0

        # Get engines
        engine_data1 = self.engines[engine1_id]
        engine_data2 = self.engines[engine2_id]
        engines_data = {}
        engines_data[0] = engine_data1
        engines_data[1] = engine_data2
        
        engine1 = Engine().get(engine1_id, engine_data1, self.db)
        engine2 = Engine().get(engine2_id, engine_data2, self.db)        
        engines = {}
        engines[0] = engine1
        engines[1] = engine2

        # Handle pairs
        pairs = json.loads(pairs_data)
        for pair in pairs:

            # Set pair config
            engine1.set_pair_config(pair[0])
            engine2.set_pair_config(pair[1])

            # Get lists
            list1 = engine1.list_items()
            list2 = engine2.list_items()

            Log.debug('lists items {0}:{1} {2}:{3}'.format(engine_data1['type'], len(list1), engine_data2['type'], len(list2)))

            # Slow sync
            if is_slowsync:
                self.slowsync(bridge_id, list1, list2, engine1, engine2)

            # Check all db
            for dbitem in data:

                # Item exists in lists
                exists1 = dbitem['item1_id'] in list1
                exists2 = dbitem['item2_id'] in list2

                # Both exist
                if exists1 and exists2:

                    # Timestamp still matches
                    if dbitem['item1_timestamp']==list1[dbitem['item1_id']].timestamp and dbitem['item2_timestamp']==list2[dbitem['item2_id']].timestamp:

                        # Do nothing
                        pass

                    else:
                        # Prepare                   
                        item_timestamp = {}
                        item_timestamp[0] = None
                        item_timestamp[1] = None

                        # Item2 has been modified
                        if dbitem['item1_timestamp']==list1[dbitem['item1_id']].timestamp:
                            engine_index = 0
                            id = dbitem['item2_id']
                            edit_id = dbitem['item1_id']

                        # Item1 has been modified    
                        else:
                            engine_index = 1
                            id = dbitem['item1_id']
                            edit_id = dbitem['item2_id']

                        # Get item and timestamp
                        item = engines[engine_index ^ 1].get_item(id)
                        item_timestamp[engine_index ^ 1] = item.timestamp

                        # Set item
                        new_item = engines[engine_index].edit_item(edit_id, item)
                        item_timestamp[engine_index] = new_item.timestamp

                        self.db.edit_data_item(dbitem['id'], item_timestamp[0], item_timestamp[1])

                        Log.debug('item edited {0}:{1} {2}'.format(engines_data[engine_index]['type'], edit_id, item.title))


                # Either of them does not exist
                else:
                    item_id = None

                    # Engine1 exists
                    if exists1:

                        engine_index = 0
                        item_id = dbitem['item1_id']

                    # Engine2 exists
                    elif exists2:

                        engine_index = 1
                        item_id = dbitem['item2_id']
    
                    # Delete
                    if item_id:

                        engines[engine_index].delete_item(item_id)

                        Log.debug('item deleted {0}:{1}'.format(engines_data[engine_index]['type'], item_id))

                    # Delete from db
                    self.db.delete_data_item(dbitem['id'])


                # Remove from lists (processed records)
                if dbitem['item1_id'] in list1:
                    del list1[dbitem['item1_id']]
                if dbitem['item2_id'] in list2:
                    del list2[dbitem['item2_id']]
        
            # Add new items
            self.add_new(list1, 0, engines, bridge_id, engine_data2)
            self.add_new(list2, 1, engines, bridge_id, engine_data1)

    def add_new(self, list, index, engines, bridge_id, engine_data):
        
        for url in list:
            
            item = {}

            # Get
            item[index] = engines[index].get_item(list[url].id)

            # Add to engine
            item[index ^ 1] = engines[index ^ 1].add_item(item[index])

            # Add to db
            self.db.add_data_item(bridge_id, item[0].id, item[0].timestamp, item[1].id, item[1].timestamp)

            Log.debug('item added {0}:{1} {2}'.format(engine_data['type'], item[index ^ 1].id, item[index].title))

    def slowsync(self, bridge_id, list1, list2, engine1, engine2):

        items1 = {}
        items2 = {}

        # Do slow only if both have items
        if len(list1) > 0 and len(list2) > 0:

            Log.debug('slow sync mode')

            # Add new items
            for url in list1:
                item = engine1.get_item(list1[url].id)
                items1[url] = item

            # Add new items
            for url in list2:
                item = engine2.get_item(list2[url].id)
                items2[url] = item

            # Find items in the list
            for url1 in items1:
                for url2 in items2:

                    item1 = items1[url1]
                    item2 = items2[url2]

                    # Match both title and body
                    if str(item1.title).strip() == str(item2.title).strip() and str(item1.body).strip() == str(item2.body).strip():

                        Log.debug('slow sync match {0} {1}'.format(item1.id, item2.id))

                        # Delete from lists
                        if url1 in list1: 
                            del list1[url1]
                        if url2 in list2:
                            del list2[url2]

                        self.db.add_data_item(bridge_id, item1.id, item1.timestamp, item2.id, item2.timestamp)

